#
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################
"""
The arg spec for the iosxr_ospfv2 module
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


class Ospfv2Args(object):  # pylint: disable=R0903
    """The arg spec for the iosxr_ospfv2 module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "config": {
            "options": {
                "processes": {
                    "elements": "dict",
                    "options": {
                        "address_family_unicast": {"type": "bool"},
                        "adjacency_stagger": {
                            "options": {
                                "disable": {"type": "bool"},
                                "max_adjacency": {"type": "int"},
                                "min_adjacency": {"type": "int"},
                            },
                            "required_together": [
                                ["min_adjacency", "max_adjacency"],
                            ],
                            "type": "dict",
                        },
                        "apply_weight": {
                            "mutually_exclusive": [
                                ["bandwidth", "default_weight"],
                            ],
                            "options": {
                                "bandwidth": {"type": "int"},
                                "default_weight": {"type": "int"},
                            },
                            "type": "dict",
                        },
                        "areas": {
                            "elements": "dict",
                            "options": {
                                "area_id": {"required": True, "type": "str"},
                                "authentication": {
                                    "mutually_exclusive": [
                                        [
                                            "keychain",
                                            "message_digest",
                                            "no_auth",
                                        ],
                                    ],
                                    "options": {
                                        "keychain": {
                                            "type": "str",
                                            "no_log": False,
                                        },
                                        "message_digest": {
                                            "options": {
                                                "keychain": {
                                                    "type": "str",
                                                    "no_log": False,
                                                },
                                            },
                                            "type": "dict",
                                        },
                                        "no_auth": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "authentication_key": {
                                    "mutually_exclusive": [
                                        ["clear", "encrypted"],
                                    ],
                                    "options": {
                                        "clear": {"type": "str"},
                                        "encrypted": {"type": "str"},
                                        "password": {
                                            "type": "str",
                                            "no_log": True,
                                        },
                                    },
                                    "type": "dict",
                                    "no_log": True,
                                },
                                "bfd": {
                                    "options": {
                                        "fast_detect": {
                                            "options": {
                                                "set": {"type": "bool"},
                                                "strict_mode": {
                                                    "type": "bool",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                        "minimum_interval": {"type": "int"},
                                        "multiplier": {"type": "int"},
                                    },
                                    "type": "dict",
                                },
                                "cost": {"type": "int"},
                                "dead_interval": {"type": "int"},
                                "default_cost": {"type": "int"},
                                "hello_interval": {"type": "int"},
                                "mpls": {
                                    "options": {
                                        "ldp": {
                                            "options": {
                                                "auto_config": {
                                                    "type": "bool",
                                                },
                                                "sync": {"type": "bool"},
                                                "sync_igp_shortcuts": {
                                                    "type": "bool",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                        "traffic_eng": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "mtu_ignore": {
                                    "choices": ["enable", "disable"],
                                    "type": "str",
                                },
                                "nssa": {
                                    "options": {
                                        "default_information_originate": {
                                            "options": {
                                                "metric": {"type": "int"},
                                                "metric_type": {"type": "int"},
                                            },
                                            "type": "dict",
                                        },
                                        "no_redistribution": {"type": "bool"},
                                        "no_summary": {"type": "bool"},
                                        "set": {"type": "bool"},
                                        "translate": {
                                            "options": {
                                                "type7": {
                                                    "options": {
                                                        "always": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                    "type": "dict",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                    },
                                    "type": "dict",
                                },
                                "ranges": {
                                    "elements": "dict",
                                    "mutually_exclusive": [
                                        ["advertise", "not_advertise"],
                                    ],
                                    "options": {
                                        "address": {
                                            "required": True,
                                            "type": "str",
                                        },
                                        "advertise": {"type": "bool"},
                                        "not_advertise": {"type": "bool"},
                                    },
                                    "type": "list",
                                },
                                "route_policy": {
                                    "elements": "dict",
                                    "options": {
                                        "direction": {
                                            "choices": ["in", "out"],
                                            "type": "str",
                                        },
                                        "parameters": {
                                            "elements": "str",
                                            "type": "list",
                                        },
                                    },
                                    "type": "list",
                                },
                                "stub": {
                                    "options": {
                                        "no_summary": {"type": "bool"},
                                        "set": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "transmit_delay": {"type": "int"},
                                "virtual_link": {
                                    "elements": "dict",
                                    "options": {
                                        "authentication": {
                                            "mutually_exclusive": [
                                                [
                                                    "keychain",
                                                    "message_digest",
                                                    "no_auth",
                                                ],
                                            ],
                                            "options": {
                                                "keychain": {
                                                    "type": "str",
                                                    "no_log": False,
                                                },
                                                "message_digest": {
                                                    "options": {
                                                        "keychain": {
                                                            "type": "str",
                                                            "no_log": False,
                                                        },
                                                    },
                                                    "type": "dict",
                                                },
                                                "no_auth": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "authentication_key": {
                                            "mutually_exclusive": [
                                                ["clear", "encrypted"],
                                            ],
                                            "options": {
                                                "clear": {"type": "str"},
                                                "encrypted": {"type": "str"},
                                                "password": {
                                                    "type": "str",
                                                    "no_log": True,
                                                },
                                            },
                                            "type": "dict",
                                            "no_log": True,
                                        },
                                        "dead_interval": {"type": "int"},
                                        "hello_interval": {"type": "int"},
                                        "id": {
                                            "required": True,
                                            "type": "str",
                                        },
                                        "message_digest_key": {
                                            "options": {
                                                "id": {
                                                    "required": True,
                                                    "type": "int",
                                                },
                                                "md5": {
                                                    "mutually_exclusive": [
                                                        ["clear", "encrypted"],
                                                    ],
                                                    "options": {
                                                        "clear": {
                                                            "type": "bool",
                                                        },
                                                        "encrypted": {
                                                            "type": "bool",
                                                        },
                                                        "password": {
                                                            "type": "str",
                                                            "no_log": True,
                                                        },
                                                    },
                                                    "type": "dict",
                                                    "no_log": False,
                                                },
                                            },
                                            "type": "dict",
                                            "no_log": False,
                                        },
                                        "retransmit_interval": {"type": "int"},
                                        "transmit_delay": {"type": "int"},
                                    },
                                    "type": "list",
                                },
                            },
                            "type": "list",
                        },
                        "authentication": {
                            "mutually_exclusive": [
                                ["keychain", "message_digest", "no_auth"],
                            ],
                            "options": {
                                "keychain": {"type": "str", "no_log": False},
                                "message_digest": {
                                    "options": {
                                        "keychain": {
                                            "type": "str",
                                            "no_log": False,
                                        },
                                        "set": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "no_auth": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "authentication_key": {
                            "mutually_exclusive": [["clear", "encrypted"]],
                            "options": {
                                "clear": {"type": "bool"},
                                "encrypted": {"type": "bool"},
                                "password": {"type": "str", "no_log": True},
                            },
                            "type": "dict",
                            "no_log": True,
                        },
                        "auto_cost": {
                            "mutually_exclusive": [
                                ["reference_bandwidth", "disable"],
                            ],
                            "options": {
                                "disable": {"type": "bool"},
                                "reference_bandwidth": {"type": "int"},
                            },
                            "type": "dict",
                        },
                        "bfd": {
                            "options": {
                                "fast_detect": {
                                    "options": {
                                        "set": {"type": "bool"},
                                        "strict_mode": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "minimum_interval": {"type": "int"},
                                "multiplier": {"type": "int"},
                            },
                            "type": "dict",
                        },
                        "capability": {
                            "options": {
                                "opaque": {
                                    "mutually_exclusive": [["set", "disable"]],
                                    "options": {
                                        "disable": {"type": "bool"},
                                        "set": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "type7": {"type": "str"},
                            },
                            "type": "dict",
                        },
                        "cost": {"type": "int"},
                        "database_filter": {
                            "choices": ["enable", "disable"],
                            "type": "str",
                        },
                        "dead_interval": {"type": "int"},
                        "default_information_originate": {
                            "options": {
                                "always": {"type": "bool"},
                                "metric": {"type": "int"},
                                "metric_type": {"type": "int"},
                                "route_policy": {"type": "str"},
                                "set": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "default_metric": {"type": "int"},
                        "demand_circuit": {
                            "choices": ["enable", "disable"],
                            "type": "str",
                        },
                        "distance": {
                            "options": {
                                "admin_distance": {
                                    "elements": "dict",
                                    "options": {
                                        "access_list": {"type": "str"},
                                        "source": {"type": "str"},
                                        "value": {"type": "int"},
                                        "wildcard": {"type": "str"},
                                    },
                                    "required_together": [
                                        ["value", "source", "wildcard"],
                                    ],
                                    "type": "list",
                                },
                                "ospf_distance": {
                                    "options": {
                                        "external": {"type": "int"},
                                        "inter_area": {"type": "int"},
                                        "intra_area": {"type": "int"},
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "distribute_bgp_ls": {
                            "options": {
                                "instance_id": {"type": "int"},
                                "throttle": {"type": "int"},
                            },
                            "type": "dict",
                        },
                        "distribute_link_state": {
                            "options": {
                                "instance_id": {"type": "int"},
                                "throttle": {"type": "int"},
                            },
                            "type": "dict",
                        },
                        "distribute_list": {
                            "elements": "dict",
                            "mutually_exclusive": [
                                ["access_list", "route_policy"],
                            ],
                            "options": {
                                "access_list": {"type": "str"},
                                "direction": {
                                    "choices": ["in", "out"],
                                    "type": "str",
                                },
                                "outgoing_params": {
                                    "options": {
                                        "id": {"type": "str"},
                                        "route_type": {
                                            "choices": [
                                                "bgp",
                                                "connected",
                                                "dagr",
                                                "ospf",
                                                "static",
                                            ],
                                            "type": "str",
                                        },
                                    },
                                    "type": "dict",
                                },
                                "route_policy": {"type": "str"},
                            },
                            "type": "list",
                        },
                        "external_out": {
                            "choices": ["enable", "disable"],
                            "type": "str",
                        },
                        "flood_reduction": {
                            "choices": ["enable", "disable"],
                            "type": "str",
                        },
                        "hello_interval": {"type": "int"},
                        "ignore_lsa_mospf": {"type": "bool"},
                        "link_down_fast_detect": {"type": "bool"},
                        "log_adjacency_changes": {
                            "options": {
                                "detail": {"type": "bool"},
                                "disable": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "loopback_stub_network": {
                            "choices": ["enable", "disable"],
                            "type": "str",
                        },
                        "max_lsa": {
                            "options": {
                                "ignore_count": {"type": "int"},
                                "ignore_time": {"type": "int"},
                                "reset_time": {"type": "int"},
                                "threshold": {"type": "int"},
                                "warning_only": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "max_metric": {
                            "options": {
                                "router_lsa": {
                                    "options": {
                                        "external_lsa": {
                                            "mutually_exclusive": [["max_metric_value", "set"]],
                                            "options": {
                                                "max_metric_value": {
                                                    "type": "int",
                                                },
                                                "set": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "include_stub": {"type": "bool"},
                                        "on_startup": {
                                            "mutually_exclusive": [["wait_for_bgp", "wait_period"]],
                                            "options": {
                                                "wait_for_bgp": {
                                                    "type": "bool",
                                                },
                                                "wait_period": {"type": "int"},
                                            },
                                            "type": "dict",
                                        },
                                        "set": {"type": "bool"},
                                        "summary_lsa": {
                                            "mutually_exclusive": [["max_metric_value", "set"]],
                                            "options": {
                                                "max_metric_value": {
                                                    "type": "int",
                                                },
                                                "set": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "message_digest_key": {
                            "options": {
                                "id": {"required": True, "type": "int"},
                                "md5": {
                                    "mutually_exclusive": [
                                        ["clear", "encrypted"],
                                    ],
                                    "options": {
                                        "clear": {"type": "bool"},
                                        "encrypted": {"type": "bool"},
                                        "password": {
                                            "type": "str",
                                            "no_log": True,
                                        },
                                    },
                                    "required": True,
                                    "type": "dict",
                                    "no_log": False,
                                },
                            },
                            "type": "dict",
                            "no_log": False,
                        },
                        "microloop_avoidance": {
                            "mutually_exclusive": [
                                [
                                    "protected",
                                    "rib_update_delay",
                                    "segment_routing",
                                ],
                            ],
                            "options": {
                                "protected": {"type": "bool"},
                                "rib_update_delay": {"type": "int"},
                                "segment_routing": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "monitor_convergence": {
                            "options": {
                                "prefix_list": {"type": "str"},
                                "track_external_routes": {"type": "bool"},
                                "track_ip_frr": {"type": "bool"},
                                "track_summary_routes": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "mpls": {
                            "options": {
                                "ldp": {
                                    "options": {
                                        "auto_config": {"type": "bool"},
                                        "sync": {"type": "bool"},
                                        "sync_igp_shortcuts": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "traffic_eng": {
                                    "options": {
                                        "autoroute_exclude": {
                                            "options": {
                                                "parameters": {
                                                    "elements": "str",
                                                    "type": "list",
                                                },
                                                "route_policy": {
                                                    "type": "str",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                        "igp_intact": {"type": "bool"},
                                        "ldp_sync_update": {"type": "bool"},
                                        "multicast_intact": {"type": "bool"},
                                        "router_id": {"type": "str"},
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "mtu_ignore": {
                            "choices": ["enable", "disable"],
                            "type": "str",
                        },
                        "network": {
                            "options": {
                                "broadcast": {"type": "bool"},
                                "non_broadcast": {"type": "bool"},
                                "point_to_multipoint": {"type": "bool"},
                                "point_to_point": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "nsf": {
                            "options": {
                                "cisco": {
                                    "options": {
                                        "enforce_global": {"type": "bool"},
                                        "set": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "flush_delay_time": {"type": "int"},
                                "ietf": {
                                    "options": {
                                        "helper_disable": {"type": "bool"},
                                        "set": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "interval": {"type": "int"},
                                "lifetime": {"type": "int"},
                            },
                            "type": "dict",
                        },
                        "nsr": {"type": "bool"},
                        "packet_size": {"type": "int"},
                        "passive": {
                            "choices": ["enable", "disable"],
                            "type": "str",
                        },
                        "prefix_suppression": {
                            "options": {
                                "secondary_address": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "priority": {"type": "int"},
                        "process_id": {"required": True, "type": "str"},
                        "protocol_shutdown": {
                            "options": {
                                "host_mode": {"type": "bool"},
                                "limit": {
                                    "options": {
                                        "high": {"type": "int"},
                                        "low": {"type": "int"},
                                        "medium": {"type": "int"},
                                    },
                                    "type": "dict",
                                },
                                "on_reload": {"type": "bool"},
                                "set": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "redistribute": {
                            "options": {
                                "id": {"type": "str"},
                                "level": {
                                    "choices": [1, 2, 12],
                                    "type": "int",
                                },
                                "lsa_type_summary": {"type": "bool"},
                                "match": {"type": "str"},
                                "metric": {"type": "int"},
                                "metric_type": {
                                    "choices": [1, 2],
                                    "type": "int",
                                },
                                "nssa_only": {"type": "bool"},
                                "preserve_med": {"type": "bool"},
                                "route_policy": {
                                    "options": {
                                        "name": {"type": "str"},
                                        "parameters": {
                                            "elements": "str",
                                            "type": "list",
                                        },
                                    },
                                    "type": "dict",
                                },
                                "route_type": {
                                    "choices": [
                                        "application",
                                        "bgp",
                                        "connected",
                                        "dagr",
                                        "eigrp",
                                        "isis",
                                        "mobile",
                                        "ospf",
                                        "rip",
                                        "static",
                                        "subscriber",
                                    ],
                                    "type": "str",
                                },
                                "tag": {"type": "int"},
                            },
                            "type": "dict",
                        },
                        "retransmit_interval": {"type": "int"},
                        "router_id": {"type": "str"},
                        "security_ttl": {
                            "options": {
                                "hops": {"type": "int"},
                                "set": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "summary_in": {
                            "choices": ["enable", "disable"],
                            "type": "str",
                        },
                        "summary_prefix": {
                            "elements": "dict",
                            "options": {
                                "not_advertise": {"type": "bool"},
                                "prefix": {"required": True, "type": "str"},
                                "tag": {"type": "int"},
                            },
                            "type": "list",
                        },
                        "timers": {
                            "options": {
                                "graceful_shutdown": {
                                    "options": {
                                        "initial_delay": {"type": "int"},
                                        "retain_routes": {"type": "int"},
                                    },
                                    "type": "dict",
                                },
                                "lsa": {
                                    "options": {
                                        "group_pacing": {"type": "int"},
                                        "min_arrival": {"type": "int"},
                                        "refresh": {"type": "int"},
                                    },
                                    "type": "dict",
                                },
                                "pacing_flood": {"type": "int"},
                                "throttle": {
                                    "options": {
                                        "fast_reroute": {"type": "int"},
                                        "lsa_all": {
                                            "options": {
                                                "initial_delay": {
                                                    "type": "int",
                                                },
                                                "max_delay": {"type": "int"},
                                                "min_delay": {"type": "int"},
                                            },
                                            "type": "dict",
                                        },
                                        "spf": {
                                            "options": {
                                                "change_delay": {
                                                    "type": "int",
                                                },
                                                "max_wait": {"type": "int"},
                                                "second_delay": {
                                                    "type": "int",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "transmit_delay": {"type": "int"},
                        "weight": {"type": "int"},
                    },
                    "type": "list",
                },
            },
            "type": "dict",
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



# Copyright (C) 2012, Michael DeHaan, <michael.dehaan@gmail.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
author: Unknown (!UNKNOWN)
name: context_demo
type: aggregate
short_description: Demo callback that adds play/task context
description:
  - Displays some play and task context along with normal output.
  - This is mostly for demo purposes.
requirements:
  - whitelist in configuration
"""

from ansible.plugins.callback import CallbackBase


class CallbackModule(CallbackBase):
    """
    This is a very trivial example of how any callback function can get at play and task objects.
    play will be 'None' for runner invocations, and task will be None for 'setup' invocations.
    """

    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "aggregate"
    CALLBACK_NAME = "community.general.context_demo"
    CALLBACK_NEEDS_WHITELIST = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.task = None
        self.play = None

    def v2_on_any(self, *args, **kwargs):
        self._display.display(f"--- play: {getattr(self.play, 'name', None)} task: {self.task} ---")

        self._display.display("     --- ARGS ")
        for i, a in enumerate(args):
            self._display.display(f"     {i}: {a}")

        self._display.display("      --- KWARGS ")
        for k in kwargs:
            self._display.display(f"     {k}: {kwargs[k]}")

    def v2_playbook_on_play_start(self, play):
        self.play = play

    def v2_playbook_on_task_start(self, task, is_conditional):
        self.task = task



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The nxos lldp_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.lldp_interfaces.lldp_interfaces import (
    Lldp_interfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.utils.utils import (
    get_interface_type,
)


class Lldp_interfacesFacts(object):
    """The nxos lldp_interfaces fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Lldp_interfacesArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_device_data(self, connection):
        return connection.get("show running-config | section ^interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for lldp_interfaces
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = self.get_device_data(connection)

        objs = []

        data = data.split("interface")
        resources = []

        for i in range(len(data)):
            intf = data[i].split("\n")
            for line in range(1, len(intf)):
                if not re.search("lldp", intf[line]):
                    intf[line] = ""
            intf = list(filter(None, intf))
            intf = "".join(i for i in intf)
            resources.append(intf)

        for resource in resources:
            if resource:  # and re.search(r'lldp', resource):
                obj = self.render_config(self.generated_spec, resource)
                if obj and len(obj.keys()) >= 1:
                    objs.append(obj)

        ansible_facts["ansible_network_resources"].pop("lldp_interfaces", None)
        facts = {}
        if objs:
            facts["lldp_interfaces"] = []
            params = utils.validate_config(self.argument_spec, {"config": objs})
            for cfg in params["config"]:
                facts["lldp_interfaces"].append(utils.remove_empties(cfg))

        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)
        match = re.search(r"^ (\S+)", conf)
        if match is None:
            return {}
        intf = match.group(1)
        if get_interface_type(intf) not in ["management", "ethernet"]:
            return {}
        config["name"] = intf
        if "lldp receive" in conf:  # for parsed state only
            config["receive"] = True
        if "no lldp receive" in conf:
            config["receive"] = False

        if "lldp transmit" in conf:  # for parsed state only
            config["transmit"] = True
        if "no lldp transmit" in conf:
            config["transmit"] = False
        if "management-address" in conf:
            config["tlv_set"]["management_address"] = re.search(
                r"management-address (\S*)",
                conf,
            ).group(1)
        if "vlan" in conf:
            config["tlv_set"]["vlan"] = re.search(r"vlan (\S*)", conf).group(1)
        return utils.remove_empties(config)



# (c) 2018 Yunge Zhu, <yungez@microsoft.com>
# (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = """
---
name: azure_service_principal_attribute

requirements:
    - msgraph-sdk

author:
    - Yunge Zhu (@yungezz)

version_added: "1.12.0"

short_description: Look up Azure service principal attributes.

description:
  - Describes object id of your Azure service principal account.
options:
  client_id:
    aliases:
      - azure_client_id
  secret:
    aliases:
      - azure_secret
  tenant:
    aliases:
      - azure_tenant
  cloud_environment:
    aliases:
      - azure_cloud_environment
notes:
    - If MSI is not enabled on ansible host, it's required to provide a valid service principal which has access to the key vault.
    - To authenticate via service principal, pass client_id, secret and tenant or set environment variables
      AZURE_CLIENT_ID, AZURE_CLIENT_SECRET and AZURE_TENANT_ID.
    - Authentication via C(az login) is also supported.

extends_documentation_fragment:
    - azure.azcollection.azure_plugin
"""

EXAMPLES = """
set_fact:
  object_id: "{{ lookup('azure_service_principal_attribute',
                         client_id=azure_client_id,
                         secret=azure_secret,
                         tenant=azure_secret) }}"
"""

RETURN = """
_raw:
  description:
    Returns object id of service principal.
"""

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMAuth
from ansible.errors import AnsibleError
from ansible.plugins.lookup import LookupBase
from ansible.module_utils._text import to_native

try:
    import asyncio
    from msgraph import GraphServiceClient
    from msgraph.generated.service_principals.service_principals_request_builder import ServicePrincipalsRequestBuilder
except ImportError:
    pass


class LookupModule(LookupBase):
    def run(self, terms, variables, **kwargs):

        self.set_options(direct=kwargs)

        auth_source = self.get_option('auth_source')
        client_id = self.get_option('client_id')
        secret = self.get_option('secret')
        tenant = self.get_option('tenant')

        # If auth_source is auto but no client_id or secret passed in switch to cli
        if auth_source == 'auto':
            if any(v is None for v in [client_id, secret, tenant]):
                auth_source = 'cli'

        auth_options = dict(
            auth_source=auth_source,
            client_id=client_id,
            secret=secret,
            tenant=tenant,
            cloud_environment=self.get_option('cloud_environment'),
            is_ad_resource=True
        )

        azure_auth = AzureRMAuth(**auth_options)

        try:
            client = GraphServiceClient(azure_auth.azure_credential_track2)

            response = asyncio.get_event_loop().run_until_complete(self.get_service_principals(client, azure_auth.credentials['client_id']))
            if not response:
                return []
            return list(response.value)[0].id.split(',')
        except Exception as ex:
            raise AnsibleError("Failed to get service principal object id: %s" % to_native(ex))
        return False

    async def get_service_principals(self, _client, app_id):
        request_configuration = ServicePrincipalsRequestBuilder.ServicePrincipalsRequestBuilderGetRequestConfiguration(
            query_parameters=ServicePrincipalsRequestBuilder.ServicePrincipalsRequestBuilderGetQueryParameters(
                filter="servicePrincipalNames/any(c:c eq '{0}')".format(app_id),
            )
        )
        return await _client.service_principals.get(request_configuration=request_configuration)



# Copyright: (c) 2021, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""Reboot action for Windows hosts

This contains the code to reboot a Windows host for use by other action plugins
in this collection. Right now it should only be used in this collection as the
interface is not final and count be subject to change.
"""

# FOR INTERNAL COLLECTION USE ONLY
# The interfaces in this file are meant for use within the ansible.windows collection
# and may not remain stable to outside uses. Changes may be made in ANY release, even a bugfix release.
# See also: https://github.com/ansible/community/issues/539#issuecomment-780839686
# Please open an issue if you have questions about this.

import base64
import datetime
import json
import random
import time
import traceback
import uuid
import typing as t

from ansible.errors import AnsibleConnectionFailure, AnsibleError
from ansible.module_utils.common.text.converters import to_text
from ansible.plugins.connection import ConnectionBase
from ansible.utils.display import Display

from ._quote import quote_pwsh


# This is not ideal but the psrp connection plugin doesn't catch all these exceptions as an AnsibleConnectionFailure.
# Until we can guarantee we are using a version of psrp that handles all this we try to handle those issues.
try:
    from requests.exceptions import (
        RequestException,
    )
except ImportError:
    RequestException = AnsibleConnectionFailure


_LOGON_UI_KEY = (
    r"HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon\AutoLogonChecked"
)

_DEFAULT_BOOT_TIME_COMMAND = (
    "(Get-CimInstance -ClassName Win32_OperatingSystem -Property LastBootUpTime)"
    ".LastBootUpTime.ToFileTime()"
)

T = t.TypeVar("T")

display = Display()


class _ReturnResultException(Exception):
    """Used to sneak results back to the return dict from an exception"""

    def __init__(self, msg, **result):
        super().__init__(msg)
        self.result = result


class _TestCommandFailure(Exception):
    """Differentiates between a connection failure and just a command assertion failure during the reboot loop"""


def reboot_host(
    task_action: str,
    connection: ConnectionBase,
    boot_time_command: str = _DEFAULT_BOOT_TIME_COMMAND,
    connect_timeout: int = 5,
    msg: str = "Reboot initiated by Ansible",
    post_reboot_delay: int = 0,
    pre_reboot_delay: int = 2,
    reboot_timeout: int = 600,
    test_command: t.Optional[str] = None,
) -> t.Dict[str, t.Any]:
    """Reboot a Windows Host.

    Used by action plugins in ansible.windows to reboot a Windows host. It
    takes in the connection plugin so it can run the commands on the targeted
    host and monitor the reboot process. The return dict will have the
    following keys set:

        changed: Whether a change occurred (reboot was done)
        elapsed: Seconds elapsed between the reboot and it coming back online
        failed: Whether a failure occurred
        unreachable: Whether it failed to connect to the host on the first cmd
        rebooted: Whether the host was rebooted

    When failed=True there may be more keys to give some information around
    the failure like msg, exception. There are other keys that might be
    returned as well but they are dependent on the failure that occurred.

    Verbosity levels used:
        2: Message when each reboot step is completed
        4: Connection plugin operations and their results
        5: Raw commands run and the results of those commands
        Debug: Everything, very verbose

    Args:
        task_action: The name of the action plugin that is running for logging.
        connection: The connection plugin to run the reboot commands on.
        boot_time_command: The command to run when getting the boot timeout.
        connect_timeout: Override the connection timeout of the connection
            plugin when polling the rebooted host.
        msg: The message to display to interactive users when rebooting the
            host.
        post_reboot_delay: Seconds to wait after sending the reboot command
            before checking to see if it has returned.
        pre_reboot_delay: Seconds to wait when sending the reboot command.
        reboot_timeout: Seconds to wait while polling for the host to come
            back online.
        test_command: Command to run when the host is back online and
            determines the machine is ready for management. When not defined
            the default command should wait until the reboot is complete and
            all pre-login configuration has completed.

    Returns:
        (Dict[str, Any]): The return result as a dictionary. Use the 'failed'
            key to determine if there was a failure or not.
    """
    result: t.Dict[str, t.Any] = {
        "changed": False,
        "elapsed": 0,
        "failed": False,
        "unreachable": False,
        "rebooted": False,
    }
    host_context = {"do_close_on_reset": True}

    # Get current boot time. A lot of tasks that require a reboot leave the WSMan stack in a bad place. Will try to
    # get the initial boot time 3 times before giving up.
    try:
        previous_boot_time = _do_until_success_or_retry_limit(
            task_action,
            connection,
            host_context,
            "pre-reboot boot time check",
            3,
            _get_system_boot_time,
            task_action,
            connection,
            boot_time_command,
        )

    except Exception as e:
        # Report a the failure based on the last exception received.
        if isinstance(e, _ReturnResultException):
            result.update(e.result)

        if isinstance(e, AnsibleConnectionFailure):
            result["unreachable"] = True
        else:
            result["failed"] = True

        result["msg"] = str(e)
        result["exception"] = traceback.format_exc()
        return result

    # Get the original connection_timeout option var so it can be reset after
    original_connection_timeout: t.Optional[float] = None
    try:
        original_connection_timeout = connection.get_option("connection_timeout")
        display.vvvv(
            f"{task_action}: saving original connection_timeout of {original_connection_timeout}"
        )
    except KeyError:
        display.vvvv(
            f"{task_action}: connection_timeout connection option has not been set"
        )

    # Initiate reboot
    # This command may be wrapped in other shells or command making it hard to detect what shutdown.exe actually
    # returned. We use this hackery to return a json that contains the stdout/stderr/rc as a structured object for our
    # code to parse and detect if something went wrong.
    reboot_command = r"""$ErrorActionPreference = 'Continue'

if ($%s) {
    Remove-Item -LiteralPath '%s' -Force -ErrorAction SilentlyContinue
}

$stdout = $null
$stderr = . { C:\Windows\System32\shutdown.exe /r /t %s /c %s | Set-Variable stdout } 2>&1 | ForEach-Object ToString

If (Get-Variable LASTEXITCODE -ErrorAction SilentlyContinue) {
    $rc = $LASTEXITCODE
} Else {
    $rc=1
}

ConvertTo-Json -Compress -InputObject @{
    stdout = (@($stdout) -join "`n")
    stderr = (@($stderr) -join "`n")
    rc = $rc
}
""" % (
        str(not test_command),
        _LOGON_UI_KEY,
        int(pre_reboot_delay),
        quote_pwsh(msg),
    )

    expected_test_result = (
        None  # We cannot have an expected result if the command is user defined
    )
    if not test_command:
        # It turns out that LogonUI will create this registry key if it does not exist when it's about to show the
        # logon prompt. Normally this is a volatile key but if someone has explicitly created it that might no longer
        # be the case. We ensure it is not present on a reboot so we can wait until LogonUI creates it to determine
        # the host is actually online and ready, e.g. no configurations/updates still to be applied.
        # We echo a known successful statement to catch issues with powershell failing to start but the rc mysteriously
        # being 0 causing it to consider a successful reboot too early (seen on ssh connections).
        expected_test_result = f"success-{uuid.uuid4()}"
        test_command = f"Get-Item -LiteralPath '{_LOGON_UI_KEY}' -ErrorAction Stop; '{expected_test_result}'"

    start = None
    try:
        _perform_reboot(task_action, connection, reboot_command)

        start = datetime.datetime.utcnow()
        result["changed"] = True
        result["rebooted"] = True

        if post_reboot_delay != 0:
            display.vv(
                f"{task_action}: waiting an additional {post_reboot_delay} seconds"
            )
            time.sleep(post_reboot_delay)

        # Keep on trying to run the last boot time check until it is successful or the timeout is raised
        display.vv(f"{task_action} validating reboot")
        _do_until_success_or_timeout(
            task_action,
            connection,
            host_context,
            "last boot time check",
            reboot_timeout,
            _check_boot_time,
            task_action,
            connection,
            host_context,
            previous_boot_time,
            boot_time_command,
            connect_timeout,
        )

        # Reset the connection plugin connection timeout back to the original
        if original_connection_timeout is not None:
            _set_connection_timeout(
                task_action,
                connection,
                host_context,
                original_connection_timeout,
            )

        # Run test command until ti is successful or a timeout occurs
        display.vv(f"{task_action} running post reboot test command")
        _do_until_success_or_timeout(
            task_action,
            connection,
            host_context,
            "post-reboot test command",
            reboot_timeout,
            _run_test_command,
            task_action,
            connection,
            test_command,
            expected=expected_test_result,
        )

        display.vv(f"{task_action}: system successfully rebooted")

    except Exception as e:
        if isinstance(e, _ReturnResultException):
            result.update(e.result)

        result["failed"] = True
        result["msg"] = str(e)
        result["exception"] = traceback.format_exc()

    if start:
        elapsed = datetime.datetime.utcnow() - start
        result["elapsed"] = elapsed.seconds

    return result


def _check_boot_time(
    task_action: str,
    connection: ConnectionBase,
    host_context: t.Dict[str, t.Any],
    previous_boot_time: int,
    boot_time_command: str,
    timeout: int,
):
    """Checks the system boot time has been changed or not"""
    display.vvvv("%s: attempting to get system boot time" % task_action)

    # override connection timeout from defaults to custom value
    if timeout:
        _set_connection_timeout(task_action, connection, host_context, timeout)

    # try and get boot time
    current_boot_time = _get_system_boot_time(
        task_action, connection, boot_time_command
    )
    if current_boot_time == previous_boot_time:
        raise _TestCommandFailure("boot time has not changed")


def _do_until_success_or_retry_limit(
    task_action: str,
    connection: ConnectionBase,
    host_context: t.Dict[str, t.Any],
    action_desc: str,
    retries: int,
    func: t.Callable[..., T],
    *args: t.Any,
    **kwargs: t.Any,
) -> t.Optional[T]:
    """Runs the function multiple times ignoring errors until the retry limit is hit"""

    def wait_condition(idx):
        return idx < retries

    return _do_until_success_or_condition(
        task_action,
        connection,
        host_context,
        action_desc,
        wait_condition,
        func,
        *args,
        **kwargs,
    )


def _do_until_success_or_timeout(
    task_action: str,
    connection: ConnectionBase,
    host_context: t.Dict[str, t.Any],
    action_desc: str,
    timeout: float,
    func: t.Callable[..., T],
    *args: t.Any,
    **kwargs: t.Any,
) -> t.Optional[T]:
    """Runs the function multiple times ignoring errors until a timeout occurs"""
    max_end_time = datetime.datetime.utcnow() + datetime.timedelta(seconds=timeout)

    def wait_condition(idx):
        return datetime.datetime.utcnow() < max_end_time

    try:
        return _do_until_success_or_condition(
            task_action,
            connection,
            host_context,
            action_desc,
            wait_condition,
            func,
            *args,
            **kwargs,
        )
    except Exception:
        raise Exception(
            "Timed out waiting for %s (timeout=%s)" % (action_desc, timeout)
        )


def _do_until_success_or_condition(
    task_action: str,
    connection: ConnectionBase,
    host_context: t.Dict[str, t.Any],
    action_desc: str,
    condition: t.Callable[[int], bool],
    func: t.Callable[..., T],
    *args: t.Any,
    **kwargs: t.Any,
) -> t.Optional[T]:
    """Runs the function multiple times ignoring errors until the condition is false"""
    fail_count = 0
    max_fail_sleep = 12
    reset_required = False
    last_error = None

    while fail_count == 0 or condition(fail_count):
        try:
            if reset_required:
                # Keep on trying the reset until it succeeds.
                _reset_connection(task_action, connection, host_context)
                reset_required = False

            else:
                res = func(*args, **kwargs)
                display.vvvvv("%s: %s success" % (task_action, action_desc))

                return res

        except Exception as e:
            last_error = e

            if not isinstance(e, _TestCommandFailure):
                # The error may be due to a connection problem, just reset the connection just in case
                reset_required = True

            # Use exponential backoff with a max timeout, plus a little bit of randomness
            random_int = random.randint(0, 1000) / 1000
            fail_sleep = 2**fail_count + random_int
            if fail_sleep > max_fail_sleep:
                fail_sleep = max_fail_sleep + random_int

            try:
                error = str(e).splitlines()[-1]
            except IndexError:
                error = str(e)

            display.vvvvv(
                "{action}: {desc} fail {e_type} '{err}', retrying in {sleep:.4} seconds...\n{tcb}".format(
                    action=task_action,
                    desc=action_desc,
                    e_type=type(e).__name__,
                    err=error,
                    sleep=fail_sleep,
                    tcb=traceback.format_exc(),
                )
            )

            fail_count += 1
            time.sleep(fail_sleep)

    if last_error:
        raise last_error

    return None


def _execute_command(
    task_action: str,
    connection: ConnectionBase,
    command: str,
) -> t.Tuple[int, str, str]:
    """Runs a command on the Windows host and returned the result"""
    display.vvvvv(f"{task_action}: running command: {command}")

    # Need to wrap the command in our PowerShell encoded wrapper. This is done to align the command input to a
    # common shell and to allow the psrp connection plugin to report the correct exit code without manually setting
    # $LASTEXITCODE for just that plugin.
    pwsh_script = command + "; If (-not $?) { If (Get-Variable LASTEXITCODE -ErrorAction SilentlyContinue) { exit $LASTEXITCODE } Else { exit 1 } }"
    encoded_command = base64.b64encode(pwsh_script.encode("utf-16-le")).decode()
    command = f"PowerShell -NoProfile -NonInteractive -ExecutionPolicy Unrestricted -EncodedCommand {encoded_command}"

    try:
        rc, stdout, stderr = connection.exec_command(
            command, in_data=None, sudoable=False
        )
    except RequestException as e:
        # The psrp connection plugin should be doing this but until we can guarantee it does we just convert it here
        # to ensure AnsibleConnectionFailure refers to actual connection errors.
        raise AnsibleConnectionFailure(f"Failed to connect to the host: {e}")

    rc = rc or 0
    stdout = to_text(stdout, errors="surrogate_or_strict").strip()
    stderr = to_text(stderr, errors="surrogate_or_strict").strip()

    display.vvvvv(
        f"{task_action}: command result - rc: {rc}, stdout: {stdout}, stderr: {stderr}"
    )

    return rc, stdout, stderr


def _get_system_boot_time(
    task_action: str,
    connection: ConnectionBase,
    boot_time_command: str,
) -> str:
    """Gets a unique identifier to represent the boot time of the Windows host"""
    display.vvvv(f"{task_action}: getting boot time")
    rc, stdout, stderr = _execute_command(task_action, connection, boot_time_command)

    if rc != 0:
        msg = f"{task_action}: failed to get host boot time info"
        raise _ReturnResultException(msg, rc=rc, stdout=stdout, stderr=stderr)

    display.vvvv(f"{task_action}: last boot time: {stdout}")
    return stdout


def _perform_reboot(
    task_action: str,
    connection: ConnectionBase,
    reboot_command: str,
    handle_abort: bool = True,
) -> None:
    """Runs the reboot command"""
    display.vv(f"{task_action}: rebooting server...")

    stdout = stderr = None
    try:
        rc, stdout, stderr = _execute_command(task_action, connection, reboot_command)

    except AnsibleConnectionFailure as e:
        # If the connection is closed too quickly due to the system being shutdown, carry on
        display.vvvv(f"{task_action}: AnsibleConnectionFailure caught and handled: {e}")
        rc = 0

    if stdout:
        try:
            reboot_result = json.loads(stdout)
        except getattr(json.decoder, "JSONDecodeError", ValueError):
            # While the reboot command should output json it may have failed for some other reason. We continue
            # reporting with that output instead
            pass
        else:
            stdout = reboot_result.get("stdout", stdout)
            stderr = reboot_result.get("stderr", stderr)
            rc = int(reboot_result.get("rc", rc))

    # Test for "A system shutdown has already been scheduled. (1190)" and handle it gracefully
    if handle_abort and (rc == 1190 or (rc != 0 and stderr and "(1190)" in stderr)):
        display.warning("A scheduled reboot was pre-empted by Ansible.")

        # Try to abort (this may fail if it was already aborted)
        rc, stdout, stderr = _execute_command(
            task_action, connection, r"C:\Windows\System32\shutdown.exe /a"
        )
        display.vvvv(
            f"{task_action}: result from trying to abort existing shutdown - rc: {rc}, stdout: {stdout}, stderr: {stderr}"
        )

        return _perform_reboot(
            task_action,
            connection,
            reboot_command,
            handle_abort=False,
        )

    elif rc == 1115:
        # "A system shutdown is in progress."
        # This means the system is already rebooting but not from something
        # we've initiated. The best we can do is wait for it to come back
        # online and hope it was a reboot and not a shutdown.
        msg = (
            "The target has reported that a reboot or shutdown is already in progress. "
            "This has been triggered by something other than Ansible but Ansible will "
            "attempt to wait for the system to come back online. If the system is "
            "shutting down instead of rebooting then this will timeout waiting for "
            "the system to come back online and report a failure. If you see this "
            "message then you should investigate the target event logs to determine "
            "what is causing the reboot/shutdown outside of Ansible."
        )
        display.warning(msg)
        return

    if rc != 0:
        msg = f"{task_action}: Reboot command failed"
        raise _ReturnResultException(msg, rc=rc, stdout=stdout, stderr=stderr)


def _reset_connection(
    task_action: str,
    connection: ConnectionBase,
    host_context: t.Dict[str, t.Any],
    ignore_errors: bool = False,
) -> None:
    """Resets the connection handling any errors"""

    def _wrap_conn_err(func, *args, **kwargs):
        try:
            func(*args, **kwargs)

        except (AnsibleError, RequestException) as e:
            if ignore_errors:
                return False

            raise AnsibleError(e)

        return True

    # While reset() should probably better handle this some connection plugins don't clear the existing connection on
    # reset() leaving resources still in use on the target (WSMan shells). Instead we try to manually close the
    # connection then call reset. If it fails once we want to skip closing to avoid a perpetual loop and just hope
    # reset() brings us back into a good state. If it's successful we still want to try it again.
    if host_context["do_close_on_reset"]:
        display.vvvv(f"{task_action}: closing connection plugin")
        try:
            success = _wrap_conn_err(connection.close)

        except Exception:
            host_context["do_close_on_reset"] = False
            raise

        host_context["do_close_on_reset"] = success

    # For some connection plugins (ssh) reset actually does something more than close so we also class that
    display.vvvv(f"{task_action}: resetting connection plugin")
    try:
        _wrap_conn_err(connection.reset)

    except AttributeError:
        # Not all connection plugins have reset so we just ignore those, close should have done our job.
        pass


def _run_test_command(
    task_action: str,
    connection: ConnectionBase,
    command: str,
    expected: t.Optional[str] = None,
) -> None:
    """Runs the user specified test command until the host is able to run it properly"""
    display.vvvv(f"{task_action}: attempting post-reboot test command")

    rc, stdout, stderr = _execute_command(task_action, connection, command)

    if rc != 0:
        msg = f"{task_action}: Test command failed - rc: {rc}, stdout: {stdout}, stderr: {stderr}"
        raise _TestCommandFailure(msg)

    if expected and expected not in stdout:
        msg = f"{task_action}: Test command failed - '{expected}' was not in stdout: {stdout}"
        raise _TestCommandFailure(msg)


def _set_connection_timeout(
    task_action: str,
    connection: ConnectionBase,
    host_context: t.Dict[str, t.Any],
    timeout: float,
) -> None:
    """Sets the connection plugin connection_timeout option and resets the connection"""
    try:
        current_connection_timeout = connection.get_option("connection_timeout")
    except KeyError:
        # Not all connection plugins implement this, just ignore the setting if it doesn't work
        return

    if timeout == current_connection_timeout:
        return

    display.vvvv(f"{task_action}: setting connect_timeout {timeout}")
    connection.set_option("connection_timeout", timeout)

    _reset_connection(task_action, connection, host_context, ignore_errors=True)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_port_schedules
short_description: Resource module for networks _switch _port _schedules
description:
  - Manage operations create, update and delete of the resource networks _switch _port
    _schedules.
  - Add a switch port schedule.
  - Delete a switch port schedule.
  - Update a switch port schedule.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  name:
    description: The name for your port schedule. Required.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  portSchedule:
    description: The schedule for switch port scheduling. Schedules are applied to
      days of the week. When it's empty, default schedule with all days of a week
      are configured. Any unspecified day in the schedule is added as a default schedule
      configuration of the day.
    suboptions:
      friday:
        description: The schedule object for Friday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      monday:
        description: The schedule object for Monday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      saturday:
        description: The schedule object for Saturday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      sunday:
        description: The schedule object for Sunday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      thursday:
        description: The schedule object for Thursday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      tuesday:
        description: The schedule object for Tuesday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
      wednesday:
        description: The schedule object for Wednesday.
        suboptions:
          active:
            description: Whether the schedule is active (true) or inactive (false)
              during the time specified between 'from' and 'to'. Defaults to true.
            type: bool
          from:
            description: The time, from '00 00' to '24 00'. Must be less than the
              time specified in 'to'. Defaults to '00 00'. Only 30 minute increments
              are allowed.
            type: str
          to:
            description: The time, from '00 00' to '24 00'. Must be greater than the
              time specified in 'from'. Defaults to '24 00'. Only 30 minute increments
              are allowed.
            type: str
        type: dict
    type: dict
  portScheduleId:
    description: PortScheduleId path parameter. Port schedule ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch createNetworkSwitchPortSchedule
    description: Complete reference of the createNetworkSwitchPortSchedule API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-switch-port-schedule
  - name: Cisco Meraki documentation for switch deleteNetworkSwitchPortSchedule
    description: Complete reference of the deleteNetworkSwitchPortSchedule API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-switch-port-schedule
  - name: Cisco Meraki documentation for switch updateNetworkSwitchPortSchedule
    description: Complete reference of the updateNetworkSwitchPortSchedule API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-port-schedule
notes:
  - SDK Method used are
    switch.Switch.create_network_switch_port_schedule,
    switch.Switch.delete_network_switch_port_schedule,
    switch.Switch.update_network_switch_port_schedule,
  - Paths used are
    post /networks/{networkId}/switch/portSchedules,
    delete /networks/{networkId}/switch/portSchedules/{portScheduleId},
    put /networks/{networkId}/switch/portSchedules/{portScheduleId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_switch_port_schedules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    name: Weekdays schedule
    networkId: string
    portSchedule:
      friday:
        active: true
        from: '9:00'
        to: '17:00'
      monday:
        active: true
        from: '9:00'
        to: '17:00'
      saturday:
        active: false
        from: 0:00
        to: '24:00'
      sunday:
        active: false
        from: 0:00
        to: '24:00'
      thursday:
        active: true
        from: '9:00'
        to: '17:00'
      tuesday:
        active: true
        from: '9:00'
        to: '17:00'
      wednesday:
        active: true
        from: '9:00'
        to: '17:00'
- name: Delete by id
  cisco.meraki.networks_switch_port_schedules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    networkId: string
    portScheduleId: string
- name: Update by id
  cisco.meraki.networks_switch_port_schedules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    name: Weekdays schedule
    networkId: string
    portSchedule:
      friday:
        active: true
        from: '9:00'
        to: '17:00'
      monday:
        active: true
        from: '9:00'
        to: '17:00'
      saturday:
        active: false
        from: 0:00
        to: '24:00'
      sunday:
        active: false
        from: 0:00
        to: '24:00'
      thursday:
        active: true
        from: '9:00'
        to: '17:00'
      tuesday:
        active: true
        from: '9:00'
        to: '17:00'
      wednesday:
        active: true
        from: '9:00'
        to: '17:00'
    portScheduleId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "networkId": "string",
      "name": "string",
      "portSchedule": {
        "monday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "tuesday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "wednesday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "thursday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "friday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "saturday": {
          "active": true,
          "from": "string",
          "to": "string"
        },
        "sunday": {
          "active": true,
          "from": "string",
          "to": "string"
        }
      }
    }
"""



#!/usr/bin/python

# Copyright (c) 2020, Silvie Chlupova <schlupov@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: copr
short_description: Manage one of the Copr repositories
version_added: 2.0.0
description: This module can enable, disable or remove the specified repository.
author: Silvie Chlupova (@schlupov) <schlupov@redhat.com>
requirements:
  - dnf
  - dnf-plugins-core
notes:
  - Supports C(check_mode).
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  host:
    description: The Copr host to work with.
    default: copr.fedorainfracloud.org
    type: str
  protocol:
    description: This indicate which protocol to use with the host.
    default: https
    type: str
  name:
    description: Copr directory name, for example C(@copr/copr-dev).
    required: true
    type: str
  state:
    description:
      - Whether to set this project as V(enabled), V(disabled), or V(absent).
    default: enabled
    type: str
    choices: [absent, enabled, disabled]
  chroot:
    description:
      - The name of the chroot that you want to enable/disable/remove in the project, for example V(epel-7-x86_64). Default
        chroot is determined by the operating system, version of the operating system, and architecture on which the module
        is run.
    type: str
  includepkgs:
    description:
      - List of packages to include in all operations.  Inverse of O(excludepkgs), DNF will exclude any package in
        the repository that does not match this list.  Matches a name or a glob.
    type: list
    elements: str
    version_added: 9.4.0
  excludepkgs:
    description: List of packages in this repository to exclude from all operations.  Matches a name or a glob.
    type: list
    elements: str
    version_added: 9.4.0
"""

EXAMPLES = r"""
- name: Enable project Test of the user schlupov
  community.general.copr:
    host: copr.fedorainfracloud.org
    state: enabled
    name: schlupov/Test
    chroot: fedora-31-x86_64

- name: Remove project integration_tests of the group copr
  community.general.copr:
    state: absent
    name: '@copr/integration_tests'

- name: Install a repo where only packages starting with "python" that do not have i386 are seen by DNF
  community.general.copr:
    name: '@sample/repo'
    chroot: fedora-rawhide-{{ ansible_facts.architecture }}
    includepkgs:
      - 'python*'
    excludepkgs:
      - '*.i386'
"""

RETURN = r"""
repo_filename:
  description: The name of the repo file in which the copr project information is stored.
  returned: success
  type: str
  sample: _copr:copr.fedorainfracloud.org:group_copr:integration_tests.repo

repo:
  description: Path to the project on the host.
  returned: success
  type: str
  sample: copr.fedorainfracloud.org/group_copr/integration_tests
"""

import os
import stat
import traceback
from urllib.error import HTTPError

try:
    import dnf
    import dnf.cli
    import dnf.repodict
    from dnf.conf import Conf

    HAS_DNF_PACKAGES = True
    DNF_IMP_ERR = None
except ImportError:
    DNF_IMP_ERR = traceback.format_exc()
    HAS_DNF_PACKAGES = False

from ansible.module_utils import distro
from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common import respawn
from ansible.module_utils.urls import open_url


def _respawn_dnf():
    if respawn.has_respawned():
        return
    system_interpreters = (
        "/usr/libexec/platform-python",
        "/usr/bin/python3",
        "/usr/bin/python",
    )
    interpreter = respawn.probe_interpreters_for_module(system_interpreters, "dnf")
    if interpreter:
        respawn.respawn_module(interpreter)


class CoprModule:
    """The class represents a copr module.

    The class contains methods that take care of the repository state of a project,
    whether the project is enabled, disabled or missing.
    """

    ansible_module = None

    def __init__(self, host, name, state, protocol, chroot=None, check_mode=False):
        self.host = host
        self.name = name
        self.state = state
        self.chroot = chroot
        self.protocol = protocol
        self.check_mode = check_mode
        if not chroot:
            self.chroot = self.chroot_conf()
        else:
            self.chroot = chroot
            self.get_base()

    @property
    def short_chroot(self):
        """str: Chroot (distribution-version-architecture) shorten to distribution-version."""
        return self.chroot.rsplit("-", 1)[0]

    @property
    def arch(self):
        """str: Target architecture."""
        chroot_parts = self.chroot.split("-")
        return chroot_parts[-1]

    @property
    def user(self):
        """str: Copr user (this can also be the name of the group)."""
        return self._sanitize_username(self.name.split("/")[0])

    @property
    def project(self):
        """str: The name of the copr project."""
        return self.name.split("/")[1]

    @classmethod
    def need_root(cls):
        """Check if the module was run as root."""
        if os.geteuid() != 0:
            cls.raise_exception("This command has to be run under the root user.")

    @classmethod
    def get_base(cls):
        """Initialize the configuration from dnf.

        Returns:
            An instance of the BaseCli class.
        """
        cls.base = dnf.cli.cli.BaseCli(Conf())
        return cls.base

    @classmethod
    def raise_exception(cls, msg):
        """Raise either an ansible exception or a python exception.

        Args:
            msg: The message to be displayed when an exception is thrown.
        """
        if cls.ansible_module:
            raise cls.ansible_module.fail_json(msg=msg, changed=False)
        raise Exception(msg)

    def _get(self, chroot):
        """Send a get request to the server to obtain the necessary data.

        Args:
            chroot: Chroot in the form of distribution-version.

        Returns:
            Info about a repository and status code of the get request.
        """
        repo_info = None
        url = f"{self.protocol}://{self.host}/coprs/{self.name}/repo/{chroot}/dnf.repo?arch={self.arch}"
        try:
            r = open_url(url)
            status_code = r.getcode()
            repo_info = r.read().decode("utf-8")
        except HTTPError as e:
            status_code = e.getcode()
        return repo_info, status_code

    def _download_repo_info(self):
        """Download information about the repository.

        Returns:
            Information about the repository.
        """
        distribution, version = self.short_chroot.split("-", 1)
        chroot = self.short_chroot
        while True:
            repo_info, status_code = self._get(chroot)
            if repo_info:
                return repo_info
            if distribution == "rhel":
                chroot = "centos-stream-8"
                distribution = "centos"
            elif distribution == "centos":
                if version == "stream-8":
                    version = "8"
                elif version == "stream-9":
                    version = "9"
                chroot = f"epel-{version}"
                distribution = "epel"
            else:
                if str(status_code) != "404":
                    self.raise_exception("This repository does not have any builds yet so you cannot enable it now.")
                else:
                    self.raise_exception(f"Chroot {self.chroot} does not exist in {self.name}")

    def _enable_repo(self, repo_filename_path, repo_content=None):
        """Write information to a repo file.

        Args:
            repo_filename_path: Path to repository.
            repo_content: Repository information from the host.

        Returns:
            True, if the information in the repo file matches that stored on the host,
            False otherwise.
        """
        if not repo_content:
            repo_content = self._download_repo_info()
        if self.ansible_module.params["includepkgs"]:
            includepkgs_value = ",".join(self.ansible_module.params["includepkgs"])
            repo_content_strip = repo_content.rstrip(
                "\n"
            )  # Python 3.11 does not allow backslash chars within f-string expressions
            repo_content = f"{repo_content_strip}\nincludepkgs={includepkgs_value}\n"
        if self.ansible_module.params["excludepkgs"]:
            excludepkgs_value = ",".join(self.ansible_module.params["excludepkgs"])
            repo_content_strip = repo_content.rstrip(
                "\n"
            )  # Python 3.11 does not allow backslash chars within f-string expressions
            repo_content = f"{repo_content_strip}\nexcludepkgs={excludepkgs_value}\n"
        if self._compare_repo_content(repo_filename_path, repo_content):
            return False
        if not self.check_mode:
            with open(repo_filename_path, "w+") as file:
                file.write(repo_content)
            os.chmod(repo_filename_path, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
        return True

    def _get_repo_with_old_id(self):
        """Try to get a repository with the old name."""
        repo_id = f"{self.user}-{self.project}"
        if repo_id in self.base.repos and "_copr" in self.base.repos[repo_id].repofile:
            file_name = self.base.repos[repo_id].repofile.split("/")[-1]
            try:
                copr_hostname = file_name.rsplit(":", 2)[0].split(":", 1)[1]
                if copr_hostname != self.host:
                    return None
                return file_name
            except IndexError:
                return file_name
        return None

    def _read_all_repos(self, repo_id=None):
        """The method is used to initialize the base variable by
        repositories using the RepoReader class from dnf.

        Args:
            repo_id: Repo id of the repository we want to work with.
        """
        reader = dnf.conf.read.RepoReader(self.base.conf, None)
        for repo in reader:
            try:
                if repo_id:
                    if repo.id == repo_id:
                        self.base.repos.add(repo)
                        break
                else:
                    self.base.repos.add(repo)
            except dnf.exceptions.ConfigError as e:
                self.raise_exception(str(e))

    def _get_copr_repo(self):
        """Return one specific repository from all repositories on the system.

        Returns:
            The repository that a user wants to enable, disable, or remove.
        """
        repo_id = f"copr:{self.host}:{self.user}:{self.project}"
        if repo_id not in self.base.repos:
            if self._get_repo_with_old_id() is None:
                return None
        return self.base.repos[repo_id]

    def _disable_repo(self, repo_filename_path):
        """Disable the repository.

        Args:
            repo_filename_path: Path to repository.

        Returns:
            False, if the repository is already disabled on the system,
            True otherwise.
        """
        self._read_all_repos()
        repo = self._get_copr_repo()
        if repo is None:
            if self.check_mode:
                return True
            self._enable_repo(repo_filename_path)
            self._read_all_repos(f"copr:{self.host}:{self.user}:{self.project}")
            repo = self._get_copr_repo()
        for repo_id in repo.cfg.sections():
            repo_content_api = self._download_repo_info()
            with open(repo_filename_path) as file:
                repo_content_file = file.read()
            if repo_content_file != repo_content_api:
                if not self.resolve_differences(repo_content_file, repo_content_api, repo_filename_path):
                    return False
            if not self.check_mode:
                self.base.conf.write_raw_configfile(
                    repo.repofile,
                    repo_id,
                    self.base.conf.substitutions,
                    {"enabled": "0"},
                )
        return True

    def resolve_differences(self, repo_content_file, repo_content_api, repo_filename_path):
        """Detect differences between the contents of the repository stored on the
        system and the information about the repository on the server.

        Args:
            repo_content_file: The contents of the repository stored on the system.
            repo_content_api: The information about the repository from the server.
            repo_filename_path: Path to repository.

        Returns:
            False, if the contents of the repo file and the information on the server match,
            True otherwise.
        """
        repo_file_lines = repo_content_file.split("\n")
        repo_api_lines = repo_content_api.split("\n")
        repo_api_lines.remove("enabled=1")
        if "enabled=0" in repo_file_lines:
            repo_file_lines.remove("enabled=0")
            if " ".join(repo_api_lines) == " ".join(repo_file_lines):
                return False
            if not self.check_mode:
                os.remove(repo_filename_path)
                self._enable_repo(repo_filename_path, repo_content_api)
        else:
            repo_file_lines.remove("enabled=1")
            if " ".join(repo_api_lines) != " ".join(repo_file_lines):
                if not self.check_mode:
                    os.remove(repo_filename_path)
                    self._enable_repo(repo_filename_path, repo_content_api)
        return True

    def _remove_repo(self):
        """Remove the required repository.

        Returns:
            True, if the repository has been removed, False otherwise.
        """
        self._read_all_repos()
        repo = self._get_copr_repo()
        if not repo:
            return False
        if not self.check_mode:
            try:
                os.remove(repo.repofile)
            except OSError as e:
                self.raise_exception(str(e))
        return True

    def run(self):
        """The method uses methods of the CoprModule class to change the state of the repository.

        Returns:
            Dictionary with information that the ansible module displays to the user at the end of the run.
        """
        self.need_root()
        state = dict()
        repo_filename = f"_copr:{self.host}:{self.user}:{self.project}.repo"
        state["repo"] = f"{self.host}/{self.user}/{self.project}"
        state["repo_filename"] = repo_filename
        repo_filename_path = f"{self.base.conf.get_reposdir}/_copr:{self.host}:{self.user}:{self.project}.repo"
        if self.state == "enabled":
            enabled = self._enable_repo(repo_filename_path)
            state["msg"] = "enabled"
            state["state"] = bool(enabled)
        elif self.state == "disabled":
            disabled = self._disable_repo(repo_filename_path)
            state["msg"] = "disabled"
            state["state"] = bool(disabled)
        elif self.state == "absent":
            removed = self._remove_repo()
            state["msg"] = "absent"
            state["state"] = bool(removed)
        return state

    @staticmethod
    def _compare_repo_content(repo_filename_path, repo_content_api):
        """Compare the contents of the stored repository with the information from the server.

        Args:
            repo_filename_path: Path to repository.
            repo_content_api: The information about the repository from the server.

        Returns:
            True, if the information matches, False otherwise.
        """
        if not os.path.isfile(repo_filename_path):
            return False
        with open(repo_filename_path) as file:
            repo_content_file = file.read()
        return repo_content_file == repo_content_api

    @staticmethod
    def chroot_conf():
        """Obtain information about the distribution, version, and architecture of the target.

        Returns:
            Chroot info in the form of distribution-version-architecture.
        """
        (distribution, version, codename) = distro.linux_distribution(full_distribution_name=False)
        base = CoprModule.get_base()
        return f"{distribution}-{version}-{base.conf.arch}"

    @staticmethod
    def _sanitize_username(user):
        """Modify the group name.

        Args:
            user: User name.

        Returns:
            Modified user name if it is a group name with @.
        """
        if user[0] == "@":
            return f"group_{user[1:]}"
        return user


def run_module():
    """The function takes care of the functioning of the whole ansible copr module."""
    module_args = dict(
        host=dict(type="str", default="copr.fedorainfracloud.org"),
        protocol=dict(type="str", default="https"),
        name=dict(type="str", required=True),
        state=dict(type="str", choices=["enabled", "disabled", "absent"], default="enabled"),
        chroot=dict(type="str"),
        includepkgs=dict(type="list", elements="str"),
        excludepkgs=dict(type="list", elements="str"),
    )
    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)
    params = module.params

    if not HAS_DNF_PACKAGES:
        _respawn_dnf()
        module.fail_json(msg=missing_required_lib("dnf"), exception=DNF_IMP_ERR)

    CoprModule.ansible_module = module
    copr_module = CoprModule(
        host=params["host"],
        name=params["name"],
        state=params["state"],
        protocol=params["protocol"],
        chroot=params["chroot"],
        check_mode=module.check_mode,
    )
    state = copr_module.run()

    info = "Please note that this repository is not part of the main distribution"

    if params["state"] == "enabled" and state["state"]:
        module.exit_json(
            changed=state["state"],
            msg=state["msg"],
            repo=state["repo"],
            repo_filename=state["repo_filename"],
            info=info,
        )
    module.exit_json(
        changed=state["state"],
        msg=state["msg"],
        repo=state["repo"],
        repo_filename=state["repo_filename"],
    )


def main():
    """Launches ansible Copr module."""
    run_module()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2022 Western Digital Corporation
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ocapi_info
version_added: 6.3.0
short_description: Manages Out-Of-Band controllers using Open Composable API (OCAPI)
description:
  - Builds OCAPI URIs locally and sends them to remote OOB controllers to get information back.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
options:
  category:
    required: true
    description:
      - Category to execute on OOB controller.
    type: str
  command:
    required: true
    description:
      - Command to execute on OOB controller.
    type: str
  baseuri:
    required: true
    description:
      - Base URI of OOB controller.
    type: str
  proxy_slot_number:
    description: For proxied inband requests, the slot number of the IOM. Only applies if O(baseuri) is a proxy server.
    type: int
  username:
    required: true
    description:
      - Username for authenticating to OOB controller.
    type: str
  password:
    required: true
    description:
      - Password for authenticating to OOB controller.
    type: str
  timeout:
    description:
      - Timeout in seconds for URL requests to OOB controller.
    default: 10
    type: int
  job_name:
    description:
      - Name of job for fetching status.
    type: str


author: "Mike Moerk (@mikemoerk)"
"""

EXAMPLES = r"""
- name: Get job status
  community.general.ocapi_info:
    category: Status
    command: JobStatus
    baseuri: "http://iom1.wdc.com"
    jobName: FirmwareUpdate
    username: "{{ username }}"
    password: "{{ password }}"
"""

RETURN = r"""
msg:
  description: Message with action result or error description.
  returned: always
  type: str
  sample: "Action was successful"

percentComplete:
  description: Percent complete of the relevant operation. Applies to O(command=JobStatus).
  returned: when supported
  type: int
  sample: 99

operationStatus:
  description: Status of the relevant operation. Applies to O(command=JobStatus). See OCAPI documentation for details.
  returned: when supported
  type: str
  sample: "Activate needed"

operationStatusId:
  description: Integer value of status (corresponds to operationStatus). Applies to O(command=JobStatus). See OCAPI documentation
    for details.
  returned: when supported
  type: int
  sample: 65540

operationHealth:
  description: Health of the operation. Applies to O(command=JobStatus). See OCAPI documentation for details.
  returned: when supported
  type: str
  sample: "OK"

operationHealthId:
  description: >-
    Integer value for health of the operation (corresponds to RV(operationHealth)). Applies to O(command=JobStatus). See OCAPI
    documentation for details.
  returned: when supported
  type: str
  sample: "OK"

details:
  description: Details of the relevant operation. Applies to O(command=JobStatus).
  returned: when supported
  type: list
  elements: str

status:
  description: Dictionary containing status information. See OCAPI documentation for details.
  returned: when supported
  type: dict
  sample:
    {
      "Details": [
        "None"
      ],
      "Health": [
        {
          "ID": 5,
          "Name": "OK"
        }
      ],
      "State": {
        "ID": 16,
        "Name": "In service"
      }
    }
"""

from urllib.parse import urljoin

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.general.plugins.module_utils._ocapi_utils import OcapiUtils

# More will be added as module features are expanded
CATEGORY_COMMANDS_ALL = {"Jobs": ["JobStatus"]}


def main():
    result = {}
    module = AnsibleModule(
        argument_spec=dict(
            category=dict(required=True),
            command=dict(required=True, type="str"),
            job_name=dict(type="str"),
            baseuri=dict(required=True, type="str"),
            proxy_slot_number=dict(type="int"),
            username=dict(required=True),
            password=dict(required=True, no_log=True),
            timeout=dict(type="int", default=10),
        ),
        supports_check_mode=True,
    )

    category = module.params["category"]
    command = module.params["command"]

    # admin credentials used for authentication
    creds = {"user": module.params["username"], "pswd": module.params["password"]}

    # timeout
    timeout = module.params["timeout"]

    base_uri = f"https://{module.params['baseuri']}"
    proxy_slot_number = module.params.get("proxy_slot_number")
    ocapi_utils = OcapiUtils(creds, base_uri, proxy_slot_number, timeout, module)

    # Check that Category is valid
    if category not in CATEGORY_COMMANDS_ALL:
        module.fail_json(msg=f"Invalid Category '{category}'. Valid Categories = {list(CATEGORY_COMMANDS_ALL.keys())}")

    # Check that the command is valid
    if command not in CATEGORY_COMMANDS_ALL[category]:
        module.fail_json(msg=f"Invalid Command '{command}'. Valid Commands = {CATEGORY_COMMANDS_ALL[category]}")

    # Organize by Categories / Commands
    if category == "Jobs":
        if command == "JobStatus":
            if module.params.get("job_name") is None:
                module.fail_json(msg="job_name required for JobStatus command.")
            job_uri = urljoin(base_uri, f"Jobs/{module.params['job_name']}")
            result = ocapi_utils.get_job_status(job_uri)

    if result["ret"] is False:
        module.fail_json(msg=to_native(result["msg"]))
    else:
        del result["ret"]
        changed = False
        session = result.get("session", dict())
        kwargs = {
            "changed": changed,
            "session": session,
            "msg": "Action was successful."
            if not module.check_mode
            else result.get("msg", "No action performed in check mode."),
        }
        result_keys = [result_key for result_key in result if result_key not in kwargs]
        for result_key in result_keys:
            kwargs[result_key] = result[result_key]
        module.exit_json(**kwargs)


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function


__metaclass__ = type
import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def get_ospf_type(afi):
    return "ospf" if afi == "ipv4" else "ospfv3"


def get_interface_type(name):
    return "GigabitEthernet" if name.startswith("GigabitEthernet") else "Loopback"


def _compute_command(cfg):
    ospf_type = get_ospf_type(cfg["address_family"]["afi"])
    type = get_interface_type(cfg["name"])
    area = cfg["address_family"]["processes"]["area"]
    pid = cfg["address_family"]["processes"]["process_id"]
    cmd = "router {0} {1} area {2} interface {3} {4}".format(
        ospf_type,
        pid,
        area["area_id"],
        type,
        cfg["name"].split(type)[1],
    )
    return cmd


def _tmplt_ospf_int_delete(config_data):
    ospf_type = get_ospf_type(config_data["afi"])
    type = get_interface_type(config_data["name"])
    area = config_data["area"]
    command = "router {0} {1} area {2} interface {3} {4}".format(
        ospf_type,
        config_data["process"],
        area["area_id"],
        type,
        config_data["name"].split(type)[1],
    )
    return command


def _tmplt_ospf_config(config_data):
    command = _compute_command(config_data)
    return command


def _tmplt_ospf_authentication_md_config(config_data):
    command = _compute_command(config_data)
    auth = config_data["address_family"]["authentication"]
    if auth["message_digest"].get("keychain"):
        command += " authentication message-digest keychain " + auth["message_digest"]["keychain"]
    return command


def _tmplt_ospf_authentication_md_set(config_data):
    command = _compute_command(config_data)
    auth = config_data["address_family"]["authentication"]
    if auth.get("message_digest") and auth["message_digest"].get("keychain"):
        command += " authentication message-digest"
    elif auth.get("null_auth"):
        command += " authentication null"
    return command


def _tmplt_ospf_authentication_key(config_data):
    command = _compute_command(config_data)
    auth = config_data["address_family"]["authentication_key"]
    if auth.get("password"):
        command += " authentication-key " + auth["password"]
    elif auth.get("encrypted"):
        command += " authentication-key encrypted " + auth["encrypted"]
    elif auth.get("clear"):
        command += " authentication-key clear " + auth["clear"]
    return command


def _tmplt_ospf_int_bfd_min_int(config_data):
    command = _compute_command(config_data)
    bfd = config_data["address_family"]["bfd"]
    if bfd.get("minimum_interval"):
        command += " bfd minimum-interval " + str(bfd["minimum_interval"])
    return command


def _tmplt_ospf_int_bfd_mult(config_data):
    command = _compute_command(config_data)
    bfd = config_data["address_family"]["bfd"]
    if bfd.get("multiplier"):
        command += " bfd multiplier " + str(bfd["multiplier"])
    return command


def _tmplt_ospf_int_bfd_fd(config_data):
    command = _compute_command(config_data)
    bfd = config_data["address_family"]["bfd"]
    if bfd.get("fast_detect") and bfd["fast_detect"].get("set"):
        command += " bfd fast-detect"
    elif bfd.get("fast_detect") and bfd["fast_detect"].get("disable"):
        command += " bfd fast-detect disable"
    elif bfd.get("fast_detect") and bfd["fast_detect"].get("strict_mode"):
        command += " bfd fast-detect strict-mode"
    return command


def _tmplt_ospf_cost_config(config_data):
    command = _compute_command(config_data)
    command += " cost " + str(config_data["address_family"]["cost"])
    return command


def _tmplt_ospf_cost_fallback_config(config_data):
    command = _compute_command(config_data)
    fallback = config_data["address_family"]["cost_fallback"]
    command += (
        " cost-fallback " + str(fallback["cost"]) + " threshold " + str(fallback["threshold"])
    )
    return command


def _tmplt_ospf_dead_int_config(config_data):
    command = _compute_command(config_data)
    command += " dead-interval " + str(
        config_data["address_family"]["dead_interval"],
    )
    return command


def _tmplt_ospf_demand_config(config_data):
    command = _compute_command(config_data)
    if config_data["address_family"]["demand_circuit"]:
        command += " demand-circuit enable"
    else:
        command += " demand-circuit disable"
    return command


class Ospf_interfacesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Ospf_interfacesTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r'''
                    ^router
                    \s(?P<ospf_type>ospf|ospfv3)
                    \s(?P<process_id>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sinterface\s(?P<name>\S+)
                    $''',
                re.VERBOSE,
            ),
            "remval": _tmplt_ospf_int_delete,
            "setval": _tmplt_ospf_config,
            "compval": "name",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "authentication.message_digest",
            "getval": re.compile(
                r"""
                ^router
                \s(?P<ospf_type>ospf|ospfv3)
                \s(?P<process_id>\S+)
                \sarea\s(?P<area_id>\S+)
                \sinterface\s(?P<name>\S+)
                \sauthentication(?P<authentication>)
                \s(?P<opt>message-digest)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_authentication_md_set,
            "compval": "address_family.authentication.message_digest",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "authentication": {
                                "set": "{{ True if authentication is defined and opt is undefined }}",
                                "message_digest": {
                                    "set": "{{ True if opt == 'message-digest' else None }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "authentication.message_digest.keychain",
            "getval": re.compile(
                r"""
                ^router
                \s(?P<ospf_type>ospf|ospfv3)
                \s(?P<process_id>\S+)
                \sarea\s(?P<area_id>\S+)
                \sinterface\s(?P<name>\S+)
                \sauthentication(?P<authentication>)
                \s(?P<message_digest>message-digest)
                \skeychain\s(?P<keychain>\S+)$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_authentication_md_config,
            "compval": "address_family.authentication.message_digest.keychain",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "authentication": {
                                "message_digest": {
                                    "keychain": "{{ keychain }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "authentication.null_auth",
            "getval": re.compile(
                r"""
                ^router
                \s(?P<ospf_type>ospf|ospfv3)
                \s(?P<process_id>\S+)
                \sarea\s(?P<area_id>\S+)
                \sinterface\s(?P<name>\S+)
                \sauthentication(?P<authentication>)
                \s(?P<opt>null)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_authentication_md_set,
            "compval": "address_family.authentication.null_auth",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "authentication": {
                                "set": "{{ True if authentication is defined and opt is undefined }}",
                                "null_auth": "{{ True if opt == 'null' else None }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "authentication_key",
            "getval": re.compile(
                r"""
                ^router
                \s(?P<ospf_type>ospf|ospfv3)
                \s(?P<process_id>\S+)
                \sarea\s(?P<area_id>\S+)
                \sinterface\s(?P<name>\S+)
                \sauthentication-key
                (\sencrypted\s(?P<encrypted>\S+))?
                (\s(?P<key>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_authentication_key,
            "compval": "address_family.authentication_key",
            "result": {
                "{{ name }}": {
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "authentication_key": {
                                "encrypted": "{{ encrypted }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd.minimum_interval",
            "getval": re.compile(
                r"""
                    ^router
                    \s(?P<ospf_type>ospf|ospfv3)
                    \s(?P<process_id>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sinterface\s(?P<name>\S+)
                    \sbfd(?P<bfd>)
                    \sminimum-interval\s(?P<minimum_interval>\d+)
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_int_bfd_min_int,
            "compval": "address_family.bfd.minimum_interval",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },

                                },
                            },
                            "bfd": {
                                "minimum_interval": "{{ minimum_interval|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd.multiplier",
            "getval": re.compile(
                r"""
                    ^router
                    \s(?P<ospf_type>ospf|ospfv3)
                    \s(?P<process_id>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sinterface\s(?P<name>\S+)
                    \sbfd(?P<bfd>)
                    \smultiplier\s(?P<multiplier>\d+)
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_int_bfd_mult,
            "compval": "address_family.bfd.multiplier",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },

                                },
                            },
                            "bfd": {
                                "multiplier": "{{ multiplier|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd.fast_detect.set",
            "getval": re.compile(
                r"""
                    ^router
                    \s(?P<ospf_type>ospf|ospfv3)
                    \s(?P<process_id>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sinterface\s(?P<name>\S+)
                    \sbfd(?P<bfd>)
                    \sfast-detect(?P<fast_detect>)
                    (\s(?P<opt>(disable|strict-mode)))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_int_bfd_fd,
            "compval": "address_family.bfd.fast_detect.set",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },

                                },
                            },
                            "bfd": {
                                "fast_detect": {
                                    "set": "{{ True if opt != 'disable' and opt != 'strict-mode' and fast_detect is defined else None }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd.fast_detect.disable",
            "getval": re.compile(
                r"""
                    ^router
                    \s(?P<ospf_type>ospf|ospfv3)
                    \s(?P<process_id>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sinterface\s(?P<name>\S+)
                    \sbfd(?P<bfd>)
                    \sfast-detect(?P<fast_detect>)
                    (\s(?P<opt>(disable|strict-mode)))?
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_int_bfd_fd,
            "compval": "address_family.bfd.fast_detect.disable",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },

                                },
                            },
                            "bfd": {
                                "fast_detect": {
                                    "disable": "{{ True if opt == 'disable' else None }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd.fast_detect.strict_mode",
            "getval": re.compile(
                r"""
                    ^router
                    \s(?P<ospf_type>ospf|ospfv3)
                    \s(?P<process_id>\S+)
                    \sarea\s(?P<area_id>\S+)
                    \sinterface\s(?P<name>\S+)
                    \sbfd(?P<bfd>)
                    \sfast-detect(?P<fast_detect>)
                    \s(?P<opt>strict-mode)
                    $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_int_bfd_fd,
            "compval": "address_family.bfd.fast_detect.strict_mode",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "bfd": {
                                "fast_detect": {
                                    "strict_mode": "{{ True if opt == 'strict-mode' else None }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "cost",
            "getval": re.compile(
                r"""
                   ^router
                   \s(?P<ospf_type>ospf|ospfv3)
                   \s(?P<process_id>\S+)
                   \sarea\s(?P<area_id>\S+)
                   \sinterface\s(?P<name>\S+)
                   \scost\s(?P<cost>\S+)$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_cost_config,
            "compval": "address_family.cost",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "cost": "{{ cost }}",
                        },
                    },
                },
            },
        },
        {
            "name": "cost_fallback",
            "getval": re.compile(
                r"""
                   ^router
                   \s(?P<ospf_type>ospf|ospfv3)
                   \s(?P<process_id>\S+)
                   \sarea\s(?P<area_id>\S+)
                   \sinterface\s(?P<name>\S+)
                   \scost-fallback\s(?P<cost>\S+)
                   \sthreshold\s(?P<threshold>\S+)
                   $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_cost_fallback_config,
            "compval": "address_family.cost_fallback",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "cost_fallback": {
                                "cost": "{{ cost }}",
                                "threshold": "{{ threshold }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "dead_interval",
            "getval": re.compile(
                r'''
                        ^router
                        \s(?P<ospf_type>ospf|ospfv3)
                        \s(?P<process_id>\S+)
                        \sarea\s(?P<area_id>\S+)
                        \sinterface\s(?P<name>\S+)
                        \sdead-interval\s(?P<dead_interval>\S+)
                        $''',
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_dead_int_config,
            "compval": "address_family.dead_interval",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "dead_interval": "{{ dead_interval }}",
                        },
                    },
                },
            },
        },
        {
            "name": "demand_circuit",
            "getval": re.compile(
                r'''
                            ^router
                            \s(?P<ospf_type>ospf|ospfv3)
                            \s(?P<process_id>\S+)
                            \sarea\s(?P<area_id>\S+)
                            \sinterface\s(?P<name>\S+)
                            \sdemand-circuit\s(?P<demand_circuit>\S+)
                            $''',
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_demand_config,
            "compval": "address_family.demand_circuit",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "demand_circuit": "{{ True if demand_circuit == 'enable' else False if demand_circuit == 'disable' else None }}",
                        },
                    },
                },
            },
        },
        {
            "name": "flood_reduction",
            "getval": re.compile(
                r'''
                                ^router
                                \s(?P<ospf_type>ospf|ospfv3)
                                \s(?P<process_id>\S+)
                                \sarea\s(?P<area_id>\S+)
                                \sinterface\s(?P<name>\S+)
                                \sflood-reduction\s(?P<flood_reduction>\S+)
                                $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "flood-reduction {{ 'enable' if flood_reduction == True else 'disable' }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "flood_reduction": "{{ True if flood_reduction == 'enable' else False if flood_reduction == 'disable' else None }}",
                        },
                    },
                },
            },
        },
        {
            "name": "hello_interval",
            "getval": re.compile(
                r'''
                            ^router
                            \s(?P<ospf_type>ospf|ospfv3)
                            \s(?P<process_id>\S+)
                            \sarea\s(?P<area_id>\S+)
                            \sinterface\s(?P<name>\S+)
                            \shello-interval\s(?P<dead_interval>\S+)
                            $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "hello-interval {{ hello_interval }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "hello_interval": "{{ hello_interval }}",
                        },
                    },
                },
            },
        },
        {
            "name": "link_down.set",
            "getval": re.compile(
                r'''
                                ^router
                                \s(?P<ospf_type>ospf|ospfv3)
                                \s(?P<process_id>\S+)
                                \sarea\s(?P<area_id>\S+)
                                \sinterface\s(?P<name>\S+)
                                \s(?P<link_down>link-down)
                                (\s(?P<disable>disable))?
                                $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "link-down",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "link_down": {
                                "set": "{{ True if link_down is defined and disable is undefined else None}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "link_down.disable",
            "getval": re.compile(
                r'''
                                    ^router
                                    \s(?P<ospf_type>ospf|ospfv3)
                                    \s(?P<process_id>\S+)
                                    \sarea\s(?P<area_id>\S+)
                                    \sinterface\s(?P<name>\S+)
                                    \s(?P<link_down>link-down)
                                    \s(?P<disable>disable)
                                    $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "link-down disable",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "link_down": {
                                "disable": "{{ True if disable is defined else None }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "message_digest_key",
            "getval": re.compile(
                r"""
                ^router
                \s(?P<ospf_type>ospf|ospfv3)
                \s(?P<process_id>\S+)
                \sarea\s(?P<area_id>\S+)
                \sinterface\s(?P<name>\S+)
                \smessage-digest-key
                \s(?P<id>\d+)
                \smd5
                \s(?P<encryption>\d)
                \s(?P<key>\S+)$""",
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface "
                      "{{ type }} {{ name }} #message-digest-key {{ message_digest_key.id }} "
                      "md5 encrypted {{ message_digest_key.encrypted}}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "message_digest_key": {
                                "id": "{{ id }}",
                                "encrypted": "{{ encryption }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mpls.set_ldp",
            "getval": re.compile(
                r'''
                                            ^router
                                            \s(?P<ospf_type>ospf|ospfv3)
                                            \s(?P<process_id>\S+)
                                            \sarea\s(?P<area_id>\S+)
                                            \sinterface\s(?P<name>\S+)
                                            \s(?P<mpls>mpls)
                                            \s(?P<ldp>set_ldp)
                                            (\s(?P<sync>sync))?
                                            $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "mpls ldp",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "mpls": {
                                "set_ldp": "{{ True if set_ldp is defined and sync is undefined else None}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mpls.ldp_sync",
            "getval": re.compile(
                r'''
                                    ^router
                                    \s(?P<ospf_type>ospf|ospfv3)
                                    \s(?P<process_id>\S+)
                                    \sarea\s(?P<area_id>\S+)
                                    \sinterface\s(?P<name>\S+)
                                    \s(?P<mpls>mpls)
                                    \s(?P<ldp>ldp)
                                    \s(?P<sync>sync)
                                    (\s(?P<disable>disable))?
                                    $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "mpls ldp sync",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "mpls": {
                                "ldp_sync": "{{ True if sync is defined and disable is undefined else None}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mpls.ldp_sync_disable",
            "getval": re.compile(
                r'''
                                        ^router
                                        \s(?P<ospf_type>ospf|ospfv3)
                                        \s(?P<process_id>\S+)
                                        \sarea\s(?P<area_id>\S+)
                                        \sinterface\s(?P<name>\S+)
                                        \s(?P<mpls>mpls)
                                        \s(?P<ldp>ldp)
                                        \s(?P<sync>sync)
                                        \s(?P<disable>disable)
                                        $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "mpls ldp sync disable",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "mpls": {
                                "ldp_sync": "{{ False if disable is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "mtu_ignore",
            "getval": re.compile(
                r'''
                                ^router
                                 \s(?P<ospf_type>ospf|ospfv3)
                                 \s(?P<process_id>\S+)
                                 \sarea\s(?P<area_id>\S+)
                                 \sinterface\s(?P<name>\S+)
                                 \smtu-ignore\s(?P<mtu_ignore>\S+)
                                $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "mtu_ignore {{ 'enable' if mtu_ignore == 'True' else 'disable' if mtu_ignore == 'False' }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "mtu_ignore": "{{ True if mtu_ignore == 'enable' else False if mtu_ignore == 'disable' else None }}",
                        },
                    },
                },
            },
        },
        {
            "name": "network",
            "getval": re.compile(
                r'''
                                ^router
                                \s(?P<ospf_type>ospf|ospfv3)
                                \s(?P<process_id>\S+)
                                \sarea\s(?P<area_id>\S+)
                                \sinterface\s(?P<name>\S+)
                                \snetwork\s(?P<network>\S+)
                                 $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "network {{ network }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "network": "{{ network }}",
                        },
                    },
                },
            },
        },
        {
            "name": "packet_size",
            "getval": re.compile(
                r'''
                                ^router
                                \s(?P<ospf_type>ospf|ospfv3)
                                \s(?P<process_id>\S+)
                                \sarea\s(?P<area_id>\S+)
                                \sinterface\s(?P<name>\S+)
                                \spacket-size\s(?P<packet_size>\S+)
                                $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "packet-size {{ packet_size }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "packet_size": "{{ packet_size }}",
                        },
                    },
                },
            },
        },
        {
            "name": "passive",
            "getval": re.compile(
                r'''
                                    ^router
                                     \s(?P<ospf_type>ospf|ospfv3)
                                     \s(?P<process_id>\S+)
                                     \sarea\s(?P<area_id>\S+)
                                     \sinterface\s(?P<name>\S+)
                                     \spassive\s(?P<passive>\S+)
                                    $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "passive {{ 'enable' if passive == 'True' else 'disable' if passive == 'False' }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "passive": "{{ True if passive == 'enable' else False if passive == 'disable' else None }}",
                        },
                    },
                },
            },
        },
        {
            "name": "prefix_suppression.disable",
            "getval": re.compile(
                r'''
                                        ^router
                                         \s(?P<ospf_type>ospf|ospfv3)
                                         \s(?P<process_id>\S+)
                                         \sarea\s(?P<area_id>\S+)
                                         \sinterface\s(?P<name>\S+)
                                         \sprefix-suppression\s(?P<prefix_suppression>\S+)
                                        $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "prefix-suppression {{ prefix_suppression }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "prefix_suppression": {
                                "disable": "{{ True if prefix_suppression == 'disable' else None }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "prefix_suppression.secondary_address",
            "getval": re.compile(
                r'''
                                        ^router
                                         \s(?P<ospf_type>ospf|ospfv3)
                                         \s(?P<process_id>\S+)
                                         \sarea\s(?P<area_id>\S+)
                                         \sinterface\s(?P<name>\S+)
                                         \s(?P<prefix_suppression>prefix-suppression)
                                         \s(?P<secondary_address>secondary-address)
                                         (\s(?P<disable>disable))?
                                        $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "prefix-suppression secondary-address {{ disable if secondary_address is False }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "prefix_suppression": {
                                "secondary_address": "{{ True if  secondary_address is defined and "
                                                     "disable is undefined else False if disable is defined else None }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "priority",
            "getval": re.compile(
                r'''
                                        ^router
                                        \s(?P<ospf_type>ospf|ospfv3)
                                        \s(?P<process_id>\S+)
                                        \sarea\s(?P<area_id>\S+)
                                        \sinterface\s(?P<name>\S+)
                                        \spriority\s(?P<priority>\d+)
                                        $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "priority {{ priority }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "priority": "{{ priority|int }}",
                        },
                    },
                },
            },
        },
        {
            "name": "retransmit_interval",
            "getval": re.compile(
                r'''
                                        ^router
                                        \s(?P<ospf_type>ospf|ospfv3)
                                        \s(?P<process_id>\S+)
                                        \sarea\s(?P<area_id>\S+)
                                        \sinterface\s(?P<name>\S+)
                                        \sretransmit-interval\s(?P<retransmit_interval>\d+)
                                        $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "retransmit-interval {{ retransmit_interval }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "retransmit_interval": "{{ retransmit_interval|int }}",
                        },
                    },
                },
            },
        },
        {
            "name": "security.ttl_hops",
            "getval": re.compile(
                r'''
                                            ^router
                                             \s(?P<ospf_type>ospf|ospfv3)
                                             \s(?P<process_id>\S+)
                                             \sarea\s(?P<area_id>\S+)
                                             \sinterface\s(?P<name>\S+)
                                             \s(?P<security>security)
                                             \s(?P<ttl>ttl)
                                             \shops\s(?P<hops>\d+)
                                            $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "security ttl hops {{ hops }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "security_ttl": {
                                "hops": "{{ hops|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "security.ttl",
            "getval": re.compile(
                r'''
                                                ^router
                                                 \s(?P<ospf_type>ospf|ospfv3)
                                                 \s(?P<process_id>\S+)
                                                 \sarea\s(?P<area_id>\S+)
                                                 \sinterface\s(?P<name>\S+)
                                                 \s(?P<security>security)
                                                 \s(?P<ttl>ttl)
                                                 (\s(?P<hops>hops))?
                                                $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "security ttl",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "security_ttl": {
                                "set": "{{ True if ttl is defined and hops is undefined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "transmit_delay",
            "getval": re.compile(
                r'''
                                    ^router
                                    \s(?P<ospf_type>ospf|ospfv3)
                                    \s(?P<process_id>\S+)
                                    \sarea\s(?P<area_id>\S+)
                                    \sinterface\s(?P<name>\S+)
                                    \stransmit-delay\s(?P<transmit_delay>\d+)
                                    $''',
                re.VERBOSE,
            ),
            "setval": "router {{ ospf_type }} {{ process_id }} area {{ area_id }} interface {{ type }} {{ name }} "
                      "transmit-delay {{ transmit_delay }}",
            "result": {
                "{{ name }}": {
                    "name": "{{ name }}",
                    "type": "{{ 'gigabitethernet' if 'GigabitEthernet' in name else 'loopback' if 'Loopback' in name }}",
                    "address_family": {
                        "{{ ospf_type }}": {
                            "afi": "{{ 'ipv4' if ospf_type == 'ospf' else 'ipv6' }}",
                            "processes": {
                                "{{ process_id }}": {
                                    "process_id": "{{ process_id }}",
                                    "area": {
                                        "area_id": "{{ area_id }}",
                                    },
                                },
                            },
                            "transmit_delay": "{{ transmit_delay|int }}",
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



# Copyright (c) 2016, Andrew Zenk <azenk@umn.edu>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
name: lastpass
author:
  - Andrew Zenk (!UNKNOWN) <azenk@umn.edu>
requirements:
  - lpass (command line utility)
  - must have already logged into LastPass
short_description: Fetch data from LastPass
description:
  - Use the lpass command line utility to fetch specific fields from LastPass.
options:
  _terms:
    description: Key from which you want to retrieve the field.
    required: true
    type: list
    elements: str
  field:
    description: Field to return from LastPass.
    default: 'password'
    type: str
"""

EXAMPLES = r"""
- name: get 'custom_field' from LastPass entry 'entry-name'
  ansible.builtin.debug:
    msg: "{{ lookup('community.general.lastpass', 'entry-name', field='custom_field') }}"
"""

RETURN = r"""
_raw:
  description: Secrets stored.
  type: list
  elements: str
"""

from subprocess import PIPE, Popen

from ansible.errors import AnsibleError
from ansible.module_utils.common.text.converters import to_bytes, to_text
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms


class LPassException(AnsibleError):
    pass


class LPass:
    def __init__(self, path="lpass"):
        self._cli_path = path

    @property
    def cli_path(self):
        return self._cli_path

    @property
    def logged_in(self):
        out, err = self._run(self._build_args("logout"), stdin="n\n", expected_rc=1)
        return err.startswith("Are you sure you would like to log out?")

    def _run(self, args, stdin=None, expected_rc=0):
        p = Popen([self.cli_path] + args, stdout=PIPE, stderr=PIPE, stdin=PIPE)
        out, err = p.communicate(to_bytes(stdin))
        rc = p.wait()
        if rc != expected_rc:
            raise LPassException(err)
        return to_text(out, errors="surrogate_or_strict"), to_text(err, errors="surrogate_or_strict")

    def _build_args(self, command, args=None):
        if args is None:
            args = []
        args = [command] + args
        args += ["--color=never"]
        return args

    def get_field(self, key, field):
        if field in ["username", "password", "url", "notes", "id", "name"]:
            out, err = self._run(self._build_args("show", [f"--{field}", key]))
        else:
            out, err = self._run(self._build_args("show", [f"--field={field}", key]))
        return out.strip()


class LookupModule(LookupBase):
    def run(self, terms, variables=None, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)
        field = self.get_option("field")

        lp = LPass()

        if not lp.logged_in:
            raise AnsibleError("Not logged into LastPass: please run 'lpass login' first")

        values = []
        for term in terms:
            values.append(lp.get_field(term, field))
        return values



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_dedicatedhost

version_added: "3.8.0"

short_description: Create, delete and update a dedicated host

description:
    - Creates, deletes, and updates a dedicated host.

options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    host_group_name:
        description:
            - The name of the host group.
        type: str
        required: true
    name:
        description:
            - The name of the dedicated host.
        required: true
        type: str
    location:
        description:
            - Valid Azure location for dedicated host. Defaults to location of resource group.
        type: str
    platform_fault_domain:
        description:
            - Fault domain of the dedicated host within a dedicated host group.
        type: int
        choices:
            - 0
    sku:
        description:
            - SKU of the dedicated host for Hardware Generation and VM family.
            - Only C(name) is required to be set
        type: dict
        suboptions:
            name:
                description:
                    - The sku name.
                type: str
    auto_replace_on_failure:
        description:
            - Specifies whether the dedicated host should be replaced automatically in case of a failure.
            - The value is defaulted to C(true) when not provided.
        default: true
        type: bool
    license_type:
        description:
            - Specifies the software license type that will be applied to the VMs deployed on the dedicated host.
        type: str
        choices:
            - 'None'
            - 'Windows_Server_Hybrid'
            - 'Windows_Server_Perpetual'
    is_restart:
        description:
            - Wether to restart the dedicated host.
        type: bool
        default: false
    state:
        description:
            - Assert the state of the host group. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - magodo (@magodo)
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create a dedicated host
  azure_rm_dedicatedhost:
    resource_group: myAzureResourceGroup
    name: mydedicatedhost
    location: eastus
    platform_fault_domain: 1
    sku:
      name: DSv3-Type1
    state: present

- name: Delete a dedicated host
  azure_rm_dedicatedhost:
    resource_group: myAzureResourceGroup
    host_group_name: myhostgroup
    name: mydedicatedhost
    state: absent
'''

RETURN = '''
state:
    description:
        - Gets a list of dedicated host.
    returned: always
    type: complex
    contains:
        host_group_name:
            description:
                - The name of the host group.
            type: str
            returned: always
            sample: myDedicatedHostGroup
        resource_group:
            description:
                - The name of the resource group.
            type: str
            returned: always
            sample: myResourceGroup
        name:
            description:
                - The dedicate hsot name.
            type: str
            returned: always
            sample: myHost
        id:
            description:
                - The dedicate host ID.
            type: str
            returned: always
            sample: "/subscriptions/{subscription-id}/resourceGroups/myResourceGroup/providers/Microsoft.Compute/HostGroups/myDedicatedHostGroup/hosts/myHost"
        location:
            description:
                - Resource location.
            type: str
            returned: always
            sample: eastus
        tags:
            description:
                - Resource tags.
            type: dict
            returned: always
            sample: {'key1': 'value1'}
        sku:
            description:
                - SKU of the dedicated host for Hardware Generation and VM family.
            type: dict
            returned: always
            sample: {'name': 'DSv3-Type4'}
        auto_replace_on_failure:
            description:
                - Specifies whether the dedicated host should be replaced automatically in case of a failure.
            type: bool
            returned: always
            sample: true
        license_type:
            description:
                - Specifies the software license type that will be applied to the VMs deployed on the dedicated host.
            type: str
            returned: always
            sample: Windows_Server_Hybrid
        provisioning_state:
            description:
                - The provisioning state, which only appears in the response.
            type: str
            returned: always
            sample: Succeeded
        platform_fault_domain:
            description:
                - Fault domain of the dedicated host within a dedicated host.
            type: str
            returned: always
            sample: 1
        virtual_machines:
            description:
                - A list of references to all virtual machines in the Dedicated Host.
            type: str
            returned: always
            sample: [{"id": "/subscriptions/subId/resourceGroups/myResourceGroup/providers/Microsoft.Compute/virtualMachines/vm1"}]
'''

from ansible.module_utils.basic import _load_params
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase, \
    normalize_location_name

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMDedicatedHost(AzureRMModuleBase):

    def __init__(self):

        _load_params()
        # define user inputs from playbook
        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            host_group_name=dict(type='str', required=True),
            name=dict(type='str', required=True),
            location=dict(type='str'),
            sku=dict(
                type='dict',
                options=dict(
                    name=dict(type='str')
                )
            ),
            platform_fault_domain=dict(type='int', choices=[0]),
            auto_replace_on_failure=dict(type='bool', default=True),
            license_type=dict(type='str', choices=['None', 'Windows_Server_Hybrid', 'Windows_Server_Perpetual']),
            state=dict(choices=['present', 'absent'], default='present', type='str'),
            is_restart=dict(type='bool', default=False)
        )

        self.results = dict(
            changed=False,
            state=dict()
        )

        self.resource_group = None
        self.host_group_name = None
        self.name = None
        self.state = None
        self.location = None
        self.tags = None
        self.sku = None
        self.platform_fault_domain = None
        self.auto_replace_on_failure = None
        self.license_type = None
        self.is_restart = None

        super(AzureRMDedicatedHost, self).__init__(self.module_arg_spec,
                                                   supports_tags=True,
                                                   supports_check_mode=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False

        # retrieve resource group to make sure it exists
        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        self.location = normalize_location_name(self.location)
        results = self.get_resource()

        if results is not None:
            if self.state == 'present':
                update_tags, self.tags = self.update_tags(results['tags'])
                if update_tags:
                    changed = True
                elif self.auto_replace_on_failure is not None and bool(self.auto_replace_on_failure) != bool(results['auto_replace_on_failure']):
                    changed = True
                elif self.platform_fault_domain and self.platform_fault_domain != results['platform_fault_domain']:
                    changed = True
                elif self.sku and self.sku['name'] != results['sku']['name']:
                    changed = True
                elif self.license_type and self.license_type != results['license_type']:
                    changed = True
                if not self.check_mode and changed:
                    results = self.update_dedicatedhost()
            else:
                changed = True
                if not self.check_mode:
                    results = self.delete_dedicatedhost()
        else:
            if self.state == 'present':
                changed = True
                if not self.check_mode:
                    results = self.create_dedicatedhost()
            else:
                self.log("The dedecated host not exist")
                changed = False

        if results is not None and self.is_restart:
            self.log("Restart the dedicated host. The operation will complete successfully once the dedicated host has restarted and is running.")
            changed = True
            if not self.check_mode:
                self.restart_dedicatedhost()

        self.results['changed'] = changed
        self.results['state'] = results
        return self.results

    def get_resource(self):
        self.log('Get host facts for {0}'.format(self.name))
        # get specific host group
        try:
            response = self.compute_client.dedicated_hosts.get(self.resource_group, self.host_group_name, self.name)
        except ResourceNotFoundError:
            return None

        return self.host_to_dict(response)

    def create_dedicatedhost(self):
        try:
            # create the dedicated host
            response = self.compute_client.dedicated_hosts.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                  host_group_name=self.host_group_name,
                                                                                  host_name=self.name,
                                                                                  parameters=dict(location=self.location,
                                                                                                  sku=self.sku,
                                                                                                  tags=self.tags,
                                                                                                  platform_fault_domain=self.platform_fault_domain,
                                                                                                  auto_replace_on_failure=self.auto_replace_on_failure,
                                                                                                  license_type=self.license_type))
            if isinstance(response, LROPoller):
                return self.host_to_dict(self.get_poller_result(response))
        except Exception as exc:
            self.fail("Error creating or updating host {0} - {1}".format(self.name, str(exc)))

    def update_dedicatedhost(self):
        try:
            # update the dedicated host
            response = self.compute_client.dedicated_hosts.begin_update(resource_group_name=self.resource_group,
                                                                        host_group_name=self.host_group_name,
                                                                        host_name=self.name,
                                                                        parameters=dict(sku=self.sku,
                                                                                        tags=self.tags,
                                                                                        platform_fault_domain=self.platform_fault_domain,
                                                                                        auto_replace_on_failure=self.auto_replace_on_failure,
                                                                                        license_type=self.license_type))
            if isinstance(response, LROPoller):
                return self.host_to_dict(self.get_poller_result(response))
        except Exception as exc:
            self.fail("Error creating or updating host {0} - {1}".format(self.name, str(exc)))

    def restart_dedicatedhost(self):
        try:
            # restart the dedicate host
            response = self.compute_client.dedicated_hosts.begin_restart(resource_group_name=self.resource_group,
                                                                         host_group_name=self.host_group_name,
                                                                         host_name=self.name)
            if isinstance(response, LROPoller):
                return self.get_poller_result(response)
        except Exception as exc:
            self.fail("Error restarting host {0} - {1}".format(self.name, str(exc)))

    def delete_dedicatedhost(self):
        try:
            # delete the dedicated host
            response = self.compute_client.dedicated_hosts.begin_delete(resource_group_name=self.resource_group,
                                                                        host_group_name=self.host_group_name,
                                                                        host_name=self.name)
            if isinstance(response, LROPoller):
                return self.get_poller_result(response)

        except Exception as exc:
            self.fail("Error deleting host {0} - {1}".format(self.name, str(exc)))

    def host_to_dict(self, host):
        result = dict(
            resource_group=self.resource_group,
            host_group_name=self.host_group_name,
            id=host.id,
            name=host.name,
            location=host.location,
            tags=host.tags,
            sku=dict(),
            auto_replace_on_failure=host.auto_replace_on_failure,
            license_type=host.license_type,
            provisioning_state=host.provisioning_state,
            platform_fault_domain=host.platform_fault_domain,
            virtual_machines=[]
        )
        if host.virtual_machines:
            result['virtual_machines'] = [dict(id=item.id) for item in host.virtual_machines]

        if host.sku:
            result['sku']['name'] = host.sku.name
        return result


def main():
    AzureRMDedicatedHost()


if __name__ == '__main__':
    main()
